#!/usr/bin/perl -w  # -*- cperl -*- #
#
#  tagreader.pm - A simple object to read the song tags from a file.
#
#  GNU MP3D - A portable(ish) MP3 server.
#
# Homepage:
#   http://www.gnump3d.org/
#
# Author:
#  Steve Kemp <steve@steve.org.uk>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
#  Steve Kemp
#  ---
#  http://www.steve.org.uk/


package gnump3d::tagreader;  # must live in tagreader.pm
require Exporter;
use strict;
use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

use gnump3d::ogginfo;           # Pure Perl OGG Vorbis tag parsing.
use gnump3d::oggtagreader;      # Simplistic comment extraction.
use gnump3d::mp3info;	        # Local copy of MP3::Info.

# set the version for version checking
my $VERSION = '$Revision: 1.11 $';


@ISA         = qw(Exporter);
@EXPORT      = qw( &getFileTags &formatFileTags );
%EXPORT_TAGS = ( );

# your exported package globals go here,
# as well as any optionally exported functions
@EXPORT_OK   = ( ) ;


sub getFileTags( $ )
{
    my ( $file ) = ( @_ );

    # TODO:
    #  Test for file within the cache first?
    #

    my %TAGS;

    if ( $file =~ /mp3$/i )
    {
	%TAGS = &getMP3Display($file);
    }
    elsif( $file =~ /ogg$/i )
    {
	%TAGS = &getOGGDisplay($file);
    }

    #
    # Make sure filename is defined..
    #
    if ( $file =~ /(.*)\/(.*)/ )
    {
	$file = $2;
    }
    if ( $file =~ /(.*)\.(.*)/ )
    {
	$file = $1;
    }

    $TAGS{ 'FILENAME' } = $file;

    # TODO:
    # Store in the cache
    #

    return( %TAGS );
}



#
#  Get the meta tags from an MP3 file.
#
sub getMP3Display($)
{
    my ( $file ) = (@_);

    my %TAGS;

    #
    # MPEG file information - get this regardless
    # of the presence of an ID3 tag or not.
    #
    my $inf         = &get_mp3info( $file );
    $TAGS{'LENGTH'} = $inf->{TIME}     || "";
    $TAGS{'BITRATE'}= $inf->{BITRATE}  || "";
    $TAGS{'SIZE'}   = $inf->{SIZE}     || "";


    #
    # Now look for tag information.
    #
    my $tag = &get_mp3tag( $file );

    # Early termination.
    if ( not defined $tag )
    {
	return( %TAGS );
    }


    #
    #  We have some tags .. so store them
    #
    $TAGS{'ARTIST'} = $tag->{ARTIST}   || "";
    $TAGS{'TITLE'}  = $tag->{TITLE}    || "";
    $TAGS{'ALBUM'}  = $tag->{ALBUM}    || "";
    $TAGS{'YEAR'}   = $tag->{YEAR}     || "";
    $TAGS{'COMMENT'}= $tag->{COMMENT}  || "";
    $TAGS{'TRACK'}  = $tag->{TRACKNUM} || "";
    $TAGS{'GENRE'}  = $tag->{GENRE}    || "";


    return( %TAGS );
}


#
#  Get the display text for an OGG Vorbis file.
#
sub getOGGDisplay($)
{
    my ($file) = (@_);

    my $reader = gnump3d::ogginfo->new($file);
    my %TAGS;

    # info
    while (my ($key, $v) = each %{$reader->info})
    {
      $TAGS{uc($key)} = $v;
    }


    my $comment = gnump3d::oggtagreader->new( );
    my %tags = $comment->getTags($file);

    if ( keys( %tags ) )
    {
	$TAGS{'ARTIST'} = $tags{'artist'}  || "";
	$TAGS{'COMMENT'}= $tags{'comment'} || "";
	$TAGS{'GENRE'}  = $tags{'genre'}   || "";
	$TAGS{'TRACK'}  = $tags{'track'}   || "";
	$TAGS{'ALBUM'}  = $tags{'album'}   || "";
	$TAGS{'TITLE'}  = $tags{'title'}   || "";
     }

    return( %TAGS );
}


sub formatFileTags( $% )
{
    my ( $format, %TAGS ) = ( @_ );

    my $keys  = 0;  # Count of tag keys in format string.
    my $found = 0;  # Count of keys expanded.

    while( $format =~ /(.*)\$([A-Z]+)(.*)/ )
    {
	my $pre  = $1;
	my $key  = $2;
	my $post = $3;

	$keys   += 1;

	#
	# Why, oh why did I not use '$TITLE' for the song title?
	#
	if( $key eq "SONGNAME" )
	{
	    $key = 'TITLE';
	}

	if (defined( $TAGS{$key} ) )
	{
	    # Do the insertion.
	    $format = $pre . $TAGS{$key} . $post;

	    #
	    # If this was a "key" tag value then we can
	    # increase the count of valid tags expanded.
	    #
	    if ( ( $key eq "ARTIST" )   ||
		 ( $key eq "TITLE" )    ||
		 ( $key eq "TRACK" )    ||
		 ( $key eq "FILENAME" ) ||
		 ( $key eq "LENGTH" )   ||
		 ( $key eq "BITRATE" )  ||
		 ( $key eq "ALBUM" ) )
	    {
	        $found += 1;
	    }
	}
	else
	{
	    $format = $pre . $post;
	}
    }

    #
    #  No real tags found - This is necessary as some tags such as file
    # length, and filesize are independent of real tags.
    #
    if ( $found < 1 )
    {
	$format = $TAGS{'FILENAME'};
    }
    if ( $found < $keys )
    {
      $format = $TAGS{'FILENAME'};
    }

    return($format);
}



#
# End of module
#
1;
